@@ -1,6 +1,7 @@
 package Search::Query::Dialect::Lucy;
-use Moo;
-extends 'Search::Query::Dialect::Native';
+use strict;
+use warnings;
+use base qw( Search::Query::Dialect::Native );
 use Carp;
 use Data::Dump qw( dump );
 use Scalar::Util qw( blessed );
@@ -13,19 +14,19 @@ use Lucy::Search::PhraseQuery;
 use Lucy::Search::RangeQuery;
 use Lucy::Search::TermQuery;
 use LucyX::Search::ProximityQuery;
-use LucyX::Search::NOTWildcardQuery;
-use LucyX::Search::WildcardQuery;
-use LucyX::Search::NullTermQuery;
-use LucyX::Search::AnyTermQuery;
+use Search::Query::Dialect::Lucy::NOTWildcardQuery;
+use Search::Query::Dialect::Lucy::WildcardQuery;
 
-use namespace::sweep;
+our $VERSION = '0.01';
 
-our $VERSION = '0.190_02';
-
-has 'wildcard'                  => ( is => 'rw', default => sub {'*'} );
-has 'fuzzify'                   => ( is => 'rw', default => sub {0} );
-has 'ignore_order_in_proximity' => ( is => 'rw', default => sub {0} );
-has 'allow_single_wildcards'    => ( is => 'rw', default => sub {0} );
+__PACKAGE__->mk_accessors(
+    qw(
+        wildcard
+        fuzzify
+        ignore_order_in_proximity
+        allow_single_wildcards
+        )
+);
 
 =head1 NAME
 
@@ -35,8 +36,8 @@ Search::Query::Dialect::Lucy - Lucy query dialect
 
  my $query = Search::Query->parser( dialect => 'Lucy' )->parse('foo');
  print $query;
- my $lucy_query = $query->as_lucy_query();
- my $hits = $lucy_searcher->hits( query => $lucy_query );
+ my $ks_query = $query->as_ks_query();
+ my $hits = $ks_searcher->hits( query => $ks_query );
 
 =head1 DESCRIPTION
 
@@ -51,9 +52,9 @@ methods are documented here.
 
 =cut
 
-=head2 BUILD
+=head2 init
 
-Sets Lucy-appropriate defaults.
+Overrides base method and sets SWISH-appropriate defaults.
 Can take the following params, also available as standard attribute
 methods.
 
@@ -94,9 +95,14 @@ not match.
 
 =cut
 
-sub BUILD {
+sub init {
     my $self = shift;
 
+    $self->SUPER::init(@_);
+
+    #carp dump $self;
+    $self->{wildcard} = '*';
+
     if ( $self->{default_field} and !ref( $self->{default_field} ) ) {
         $self->{default_field} = [ $self->{default_field} ];
     }
@@ -151,7 +157,7 @@ sub _doctor_value {
 
     my $value = $clause->{value};
 
-    if ( defined $value and $self->fuzzify ) {
+    if ( $self->fuzzify ) {
         $value .= '*' unless $value =~ m/[\*]/;
     }
 
@@ -212,22 +218,14 @@ sub stringify_clause {
     }
 
     # make sure we have a field
-    my $default_field
+    my $default_field 
         = $self->default_field
         || $self->parser->default_field
         || undef;    # not empty string or 0
-    my @fields;
-    if ( $clause->{field} ) {
-        @fields = ( $clause->{field} );
-    }
-    elsif ( defined $default_field ) {
-        if ( ref $default_field ) {
-            @fields = @$default_field;
-        }
-        else {
-            @fields = ($default_field);
-        }
-    }
+    my @fields
+        = $clause->{field}
+        ? ( $clause->{field} )
+        : ( defined $default_field ? @$default_field : () );
 
     # what value
     my $value
@@ -250,7 +248,7 @@ sub stringify_clause {
     if ( $prefix eq '-' ) {
         $op = '!' . $op unless $op =~ m/^!/;
     }
-    if ( defined $value and $value =~ m/[\*\?]|\Q$wildcard/ ) {
+    if ( $value =~ m/[\*\?]|\Q$wildcard/ ) {
         $op =~ s/:/~/g;
         if ( $value eq '*' or $value eq '?' ) {
             if ( !$self->allow_single_wildcards ) {
@@ -261,20 +259,16 @@ sub stringify_clause {
 
     my @buf;
 NAME: for my $name (@fields) {
-        my $field = $self->get_field($name);
+        my $field = $self->_get_field($name);
 
         if ( defined $field->callback ) {
             push( @buf, $field->callback->( $field, $op, $value ) );
             next NAME;
         }
 
-        ( $self->debug > 1 )
-            and warn "lucy string: "
-            . dump [ $name, $op, $prefix, $quote, $value, ];
-
-        if ( !defined $value ) {
-            $value = 'NULL';
-        }
+        $self->debug
+            and warn "ks string: "
+            . dump [ $name, $op, $prefix, $quote, $value ];
 
         # invert fuzzy
         if ( $op eq '!~' ) {
@@ -296,20 +290,12 @@ NAME: for my $name (@fields) {
 
         # invert
         elsif ( $op eq '!:' ) {
-
-            # double negative
-            if ( $prefix eq '-' and $clause->{op} eq '!:' ) {
-                push @buf,
-                    join( '', $name, ':', qq/$quote$value$quote$proximity/ );
-            }
-            else {
-                push(
-                    @buf,
-                    join( '',
-                        '(NOT ', $name, ':', qq/$quote$value$quote$proximity/,
-                        ')' )
-                );
-            }
+            push(
+                @buf,
+                join( '',
+                    '(NOT ', $name, ':', qq/$quote$value$quote$proximity/,
+                    ')' )
+            );
         }
 
         # range
@@ -352,7 +338,7 @@ NAME: for my $name (@fields) {
         . ( scalar(@buf) > 1 ? ')' : '' );
 }
 
-=head2 as_lucy_query
+=head2 as_ks_query
 
 Returns the Dialect object as a Lucy::Search::Query-based object.
 The Dialect object is walked and converted to a
@@ -360,53 +346,47 @@ Lucy::Searcher-compatible tree.
 
 =cut
 
-my %lucy_class_map = (
+my %ks_class_map = (
     '+' => 'AND',
     ''  => 'OR',
     '-' => 'NOT',
 );
 
-sub as_lucy_query {
+sub as_ks_query {
     my $self = shift;
     my $tree = shift || $self;
 
     my @q;
     foreach my $prefix ( '+', '', '-' ) {
         my @clauses;
-        my $joiner = $lucy_class_map{$prefix};
+        my $joiner = $ks_class_map{$prefix};
         next unless exists $tree->{$prefix};
         my $has_explicit_fields = 0;
         for my $clause ( @{ $tree->{$prefix} } ) {
-            my @lucy_clauses = $self->_lucy_clause( $clause, $prefix );
-            if ( !@lucy_clauses ) {
-
-                #warn "No lucy_clauses for $clause";
-                next;
-            }
-            push( @clauses, @lucy_clauses );
+            push( @clauses, $self->_ks_clause( $clause, $prefix ) );
             if ( defined $clause->{field} ) {
                 $has_explicit_fields++;
             }
         }
         next if !@clauses;
 
-        my $lucy_class = 'Lucy::Search::' . $joiner . 'Query';
-        my $lucy_param_name = $joiner eq 'NOT' ? 'negated_query' : 'children';
+        my $ks_class = 'Lucy::Search::' . $joiner . 'Query';
+        my $ks_param_name = $joiner eq 'NOT' ? 'negated_query' : 'children';
         @clauses = grep {defined} @clauses;
         if ( $prefix eq '-' and @clauses > 1 ) {
-            $lucy_class      = 'Lucy::Search::ANDQuery';
-            $lucy_param_name = 'children';
+            $ks_class      = 'Lucy::Search::ANDQuery';
+            $ks_param_name = 'children';
         }
 
-        #warn "$lucy_class -> new( $lucy_param_name => " . dump \@clauses;
+        #warn "$ks_class -> new( $ks_param_name => " . dump \@clauses;
         #warn "has_explicit_fields=$has_explicit_fields";
 
         if ( @clauses == 1 ) {
             if (    $prefix eq '-'
                 and $has_explicit_fields
-                and !$clauses[0]->isa($lucy_class) )
+                and !$clauses[0]->isa($ks_class) )
             {
-                push @q, $lucy_class->new( $lucy_param_name => $clauses[0] );
+                push @q, $ks_class->new( $ks_param_name => $clauses[0] );
             }
             else {
                 push @q, $clauses[0];
@@ -414,30 +394,25 @@ sub as_lucy_query {
         }
         elsif ( !$has_explicit_fields and $prefix eq '-' ) {
 
-            warn "do not wrap \@clauses in a $lucy_class";
+            warn "do not wrap \@clauses in a $ks_class";
             push @q, @clauses;
 
         }
         else {
-            push @q, $lucy_class->new( $lucy_param_name => \@clauses );
+            push @q, $ks_class->new( $ks_param_name => \@clauses );
         }
 
     }
 
-    # possible we end up with nothing if StopFilter applied
-    if ( !@q ) {
-        return ();
-    }
-
-    my $clause_joiner     = $self->_get_clause_joiner;
-    my $lucy_class_joiner = 'Lucy::Search::' . $clause_joiner . 'Query';
+    my $clause_joiner   = $self->_get_clause_joiner;
+    my $ks_class_joiner = 'Lucy::Search::' . $clause_joiner . 'Query';
 
     return @q == 1
         ? $q[0]
-        : $lucy_class_joiner->new( children => \@q );
+        : $ks_class_joiner->new( children => \@q );
 }
 
-sub _lucy_clause {
+sub _ks_clause {
     my $self   = shift;
     my $clause = shift;
     my $prefix = shift;
@@ -446,23 +421,15 @@ sub _lucy_clause {
     #warn "prefix = '$prefix'";
 
     if ( $clause->{op} eq '()' ) {
-        return $self->as_lucy_query( $clause->{value} );
+        return $self->as_ks_query( $clause->{value} );
     }
 
     # make sure we have a field
     my $default_field = $self->default_field || $self->parser->default_field;
-    my @fields;
-    if ( $clause->{field} ) {
-        @fields = ( $clause->{field} );
-    }
-    elsif ( defined $default_field ) {
-        if ( ref $default_field ) {
-            @fields = @$default_field;
-        }
-        else {
-            @fields = ($default_field);
-        }
-    }
+    my @fields
+        = $clause->{field}
+        ? ( $clause->{field} )
+        : ( defined $default_field ? @$default_field : () );
 
     # what value
     my $value
@@ -470,7 +437,7 @@ sub _lucy_clause {
         ? $clause->{value}
         : $self->_doctor_value($clause);
 
-    # if we have no fields, we can't proceed, because Lucy
+    # if we have no fields, we can't proceed, because KS
     # requires a field for every term.
     if ( !@fields ) {
         croak
@@ -485,7 +452,7 @@ sub _lucy_clause {
     if ( $prefix eq '-' ) {
         $op = '!' . $op unless $op =~ m/^!/;
     }
-    if ( defined $value and $value =~ m/[\*\?]|\Q$wildcard/ ) {
+    if ( $value =~ m/[\*\?]|\Q$wildcard/ ) {
         $op =~ s/:/~/;
         if ( $value eq '*' or $value eq '?' ) {
             if ( !$self->allow_single_wildcards ) {
@@ -500,67 +467,16 @@ sub _lucy_clause {
 
     my @buf;
 FIELD: for my $name (@fields) {
-        my $field = $self->get_field($name);
+        my $field = $self->_get_field($name);
 
         if ( defined $field->callback ) {
             push( @buf, $field->callback->( $field, $op, $value ) );
             next FIELD;
         }
 
-        if ( $self->debug ) {
-            warn "as_lucy_query:\n";
-            warn dump(
-                {   name      => $name,
-                    op        => $op,
-                    prefix    => $prefix,
-                    quote     => $quote,
-                    value     => $value,
-                    clause_op => $clause->{op},
-                }
-            ) . "\n";
-        }
-
-        # NULL
-        if ( !defined $value ) {
-            if ( $op eq '!:' ) {
-                if ( $prefix eq '-' ) {
-
-                    # original op was inverted above by $prefix
-                    if ( $clause->{op} eq ':' ) {
-
-                        # appears to be a double negative, but necessary
-                        # to get the logic and serialization correct.
-                        # e.g. NOT foo:NULL
-                        push @buf,
-                            Lucy::Search::NOTQuery->new(
-                            negated_query =>
-                                $field->nullterm_query_class->new(
-                                field => $name
-                                )
-                            );
-
-                    }
-                    else {
-                        # true double negative
-                        # e.g. NOT foo!:NULL  => foo:NULL
-                        push @buf,
-                            $field->nullterm_query_class->new(
-                            field => $name );
-
-                    }
-
-                }
-                else {
-                    push @buf,
-                        $field->anyterm_query_class->new( field => $name );
-                }
-            }
-            else {
-                push @buf,
-                    $field->nullterm_query_class->new( field => $name );
-            }
-            next FIELD;
-        }
+        $self->debug
+            and warn "as_ks_query: "
+            . dump [ $name, $op, $prefix, $quote, $value ];
 
         # range is un-analyzed
         if ( $op eq '..' ) {
@@ -568,7 +484,7 @@ FIELD: for my $name (@fields) {
                 croak "range of values must be a 2-element ARRAY";
             }
 
-            my $range_query = $field->range_query_class->new(
+            my $range_query = Lucy::Search::RangeQuery->new(
                 field         => $name,
                 lower_term    => $value->[0],
                 upper_term    => $value->[1],
@@ -587,7 +503,7 @@ FIELD: for my $name (@fields) {
                 croak "range of values must be a 2-element ARRAY";
             }
 
-            my $range_query = $field->range_query_class->new(
+            my $range_query = Lucy::Search::RangeQuery->new(
                 field         => $name,
                 lower_term    => $value->[0],
                 upper_term    => $value->[1],
@@ -600,7 +516,7 @@ FIELD: for my $name (@fields) {
             next FIELD;
         }
 
-        #$self->debug and warn "value before:$value";
+        $self->debug and warn "value before:$value";
         my @values = ($value);
 
         # if the field has an analyzer, use it on $value
@@ -613,7 +529,6 @@ FIELD: for my $name (@fields) {
                 # and strip the wildcards off.
 
                 # assume CaseFolder
-                # TODO do not assume
                 $value = lc($value);
 
                 # split on whitespace, not token regex
@@ -621,20 +536,16 @@ FIELD: for my $name (@fields) {
 
                 # if stemmer, apply only to prefix if at all.
                 my $stemmer;
-
                 if ( $field->analyzer->isa('Lucy::Analysis::PolyAnalyzer') ) {
                     my $analyzers = $field->analyzer->get_analyzers();
                     for my $ana (@$analyzers) {
-                        if ( $ana->isa('Lucy::Analysis::SnowballStemmer') ) {
+                        if ( $ana->isa('Lucy::Analysis::Stemmer') ) {
                             $stemmer = $ana;
                             last;
                         }
-
                     }
                 }
-                elsif (
-                    $field->analyzer->isa('Lucy::Analysis::SnowballStemmer') )
-                {
+                elsif ( $field->analyzer->isa('Lucy::Analysis::Stemmer') ) {
                     $stemmer = $field->analyzer;
                 }
 
@@ -656,23 +567,10 @@ FIELD: for my $name (@fields) {
             else {
                 @values = grep { defined and length }
                     @{ $field->analyzer->split($value) };
-
-                # if we have a StopFilter in our analyzer chain,
-                # it's possible the @values ends up empty.
-                # The Lucy QueryParser will handle that case
-                # by trying twice with a NoMatchQuery.
-                # TODO what should our behavior be?
-                # a stopword will never match since the word
-                # won't be in the index...
             }
         }
 
-        #$self->debug and warn "value after :" . dump( \@values );
-
-        # StopFilter or similar case
-        if ( !@values ) {
-            return ();
-        }
+        $self->debug and warn "value after :" . dump( \@values );
 
         if ( $is_phrase or @values > 1 ) {
             if ($proximity) {
@@ -683,7 +581,7 @@ FIELD: for my $name (@fields) {
                     while ( $n_values-- > 0 ) {
                         push(
                             @permutations,
-                            $field->proximity_query_class->new(
+                            LucyX::Search::ProximityQuery->new(
                                 field  => $name,
                                 terms  => [@values],    # new array
                                 within => $proximity,
@@ -704,7 +602,7 @@ FIELD: for my $name (@fields) {
                 else {
                     push(
                         @buf,
-                        $field->proximity_query_class->new(
+                        LucyX::Search::ProximityQuery->new(
                             field  => $name,
                             terms  => \@values,
                             within => $proximity,
@@ -713,30 +611,13 @@ FIELD: for my $name (@fields) {
                 }
             }
             else {
-                # invert
-                if ( $op eq '!:' ) {
-                    push(
-                        @buf,
-                        Lucy::Search::NOTQuery->new(
-                            negated_query => $field->phrase_query_class->new(
-                                field => $name,
-                                terms => \@values,
-                            )
-                        )
-                    );
-                }
-
-                # standard
-                else {
-
-                    push(
-                        @buf,
-                        $field->phrase_query_class->new(
-                            field => $name,
-                            terms => \@values,
-                        )
-                    );
-                }
+                push(
+                    @buf,
+                    Lucy::Search::PhraseQuery->new(
+                        field => $name,
+                        terms => \@values,
+                    )
+                );
             }
         }
         else {
@@ -744,7 +625,6 @@ FIELD: for my $name (@fields) {
 
             # TODO why would this happen?
             if ( !defined $term or !length $term ) {
-                warn "No term defined";
                 next FIELD;
             }
 
@@ -754,36 +634,39 @@ FIELD: for my $name (@fields) {
             {
                 $term .= $wildcard unless $term =~ m/\Q$wildcard/;
 
+                # instead of a NOTWildcardQuery, wrap the WildcardQuery
+                # in a NOTQuery. This is for matching things like:
+                #
+                #  somefield!=?*
+                #
+                # where a NOTWildcardQuery would naturally only look
+                # at terms that exist in the lexicon, and not terms
+                # that do not.
+                #
+
                 # if the prefix is already NOT do not apply a double negative
                 if ( $prefix eq '-' ) {
                     push(
                         @buf,
-                        $field->wildcard_query_class->new(
+                        Search::Query::Dialect::Lucy::WildcardQuery->new(
                             field => $name,
                             term  => $term,
                         )
                     );
                 }
-                elsif ( $op =~ m/^\!/ ) {
-                    push(
-                        @buf,
-                        Lucy::Search::NOTQuery->new(
-                            negated_query =>
-                                $field->wildcard_query_class->new(
-                                field => $name,
-                                term  => $term,
-                                )
-                        )
-                    );
-                }
                 else {
                     push(
                         @buf,
-                        $field->wildcard_query_class->new(
-                            field => $name,
-                            term  => $term,
+                        Lucy::Search::NOTQuery->new(
+                            negated_query =>
+                                Search::Query::Dialect::Lucy::WildcardQuery
+                                ->new(
+                                field => $name,
+                                term  => $term,
+                                )
                         )
                     );
+
                 }
             }
 
@@ -795,7 +678,7 @@ FIELD: for my $name (@fields) {
 
                 push(
                     @buf,
-                    $field->wildcard_query_class->new(
+                    Search::Query::Dialect::Lucy::WildcardQuery->new(
                         field => $name,
                         term  => $term,
                     )
@@ -807,7 +690,7 @@ FIELD: for my $name (@fields) {
                 push(
                     @buf,
                     Lucy::Search::NOTQuery->new(
-                        negated_query => $field->term_query_class->new(
+                        negated_query => Lucy::Search::TermQuery->new(
                             field => $name,
                             term  => $term,
                         )
@@ -819,7 +702,7 @@ FIELD: for my $name (@fields) {
             else {
                 push(
                     @buf,
-                    $field->term_query_class->new(
+                    Lucy::Search::TermQuery->new(
                         field => $name,
                         term  => $term,
                     )
@@ -828,17 +711,12 @@ FIELD: for my $name (@fields) {
 
         }    # TERM
     }
-
-    # possible that stop_filter results in nothing in buffer
-    if ( !@buf ) {
-        return ();
-    }
     if ( @buf == 1 ) {
         return $buf[0];
     }
     my $joiner = $prefix eq '-' ? 'AND' : 'OR';
-    my $lucy_class = 'Lucy::Search::' . $joiner . 'Query';
-    return $lucy_class->new( children => \@buf );
+    my $ks_class = 'Lucy::Search::' . $joiner . 'Query';
+    return $ks_class->new( children => \@buf );
 }
 
 =head2 field_class
