@@ -1,29 +1,12 @@
 package Search::Query::Field::Lucy;
-use Moo;
-extends 'Search::Query::Field';
+use strict;
+use warnings;
+use base qw( Search::Query::Field );
 use Scalar::Util qw( blessed );
 
-use namespace::sweep;
+__PACKAGE__->mk_accessors(qw( type is_int analyzer ));
 
-has 'type'   => ( is => 'rw', default => sub {'char'} );
-has 'is_int' => ( is => 'rw', default => sub {0} );
-has 'analyzer' => ( is => 'rw' );    # TODO isa check
-has 'range_query_class' =>
-    ( is => 'rw', default => sub {'Lucy::Search::RangeQuery'} );
-has 'term_query_class' =>
-    ( is => 'rw', default => sub {'Lucy::Search::TermQuery'} );
-has 'phrase_query_class' =>
-    ( is => 'rw', default => sub {'Lucy::Search::PhraseQuery'} );
-has 'proximity_query_class' =>
-    ( is => 'rw', default => sub {'LucyX::Search::ProximityQuery'} );
-has 'wildcard_query_class' =>
-    ( is => 'rw', default => sub {'LucyX::Search::WildcardQuery'} );
-has 'nullterm_query_class' =>
-    ( is => 'rw', default => sub {'LucyX::Search::NullTermQuery'} );
-has 'anyterm_query_class' =>
-    ( is => 'rw', default => sub {'LucyX::Search::AnyTermQuery'} );
-
-our $VERSION = '0.190_02';
+our $VERSION = '0.01';
 
 =head1 NAME
 
@@ -46,7 +29,7 @@ validation and aliasing in Lucy search queries.
 This class is a subclass of Search::Query::Field. Only new or overridden
 methods are documented here.
 
-=head2 BUILD
+=head2 init
 
 Available params are also standard attribute accessor methods.
 
@@ -63,49 +46,20 @@ Set if C<type> matches m/int|num|date/.
 
 =item analyzer
 
-Set to a L<Lucy::Analysis::Analyzer>-based object (optional).
-
-=item range_query_class
-
-Defaults to L<Lucy::Search::RangeQuery>.
-
-=item term_query_class
-
-Defaults to L<Lucy::Search::TermQuery>.
-
-=item phrase_query_class
-
-Defaults to L<Lucy::Search::PhraseQuery>.
-
-=item proximity_query_class
-
-Defaults to L<LucyX::Search::ProximityQuery>.
-
-=item wildcard_query_class
-
-Defaults to L<LucyX::Search::WildcardQuery>.
-
-=item nullterm_query_class
-
-Defaults to L<LucyX::Search::NullTermQuery>.
-
-=item anyterm_query_class
-
-Defaults to L<LucyX::Search::AnyTermQuery>.
+Set to a Lucy::Analysis::Analyzer-based object (optional).
 
 =back
 
 =cut
 
-sub BUILD {
+sub init {
     my $self = shift;
+    $self->SUPER::init(@_);
+
+    $self->{type} ||= 'char';
 
     # numeric types
-    if (   exists $self->{type}
-        && defined $self->{type}
-        && !blessed( $self->{type} )
-        && $self->{type} =~ m/int|date|num/ )
-    {
+    if ( !blessed( $self->{type} ) && $self->{type} =~ m/int|date|num/ ) {
         $self->{is_int} = 1;
     }
 
